@@ -43,7 +43,7 @@ def register(name, environment):
     environments[name] = environment
 
 
-def evaluate(environment, agents=[], configuration={}, steps=[], num_episodes=1):
+def evaluate(environment, agents=[], configuration={}, steps=[], num_episodes=1, debug=False, state=None):
     """
     Evaluate and return the rewards of one or more episodes (environment and agents combo).
 
@@ -53,11 +53,13 @@ def evaluate(environment, agents=[], configuration={}, steps=[], num_episodes=1)
         configuration (dict, optional):
         steps (list, optional):
         num_episodes (int=1, optional): How many episodes to execute (run until done).
+        debug (bool=False, optional): Render print() statments to stdout
+        state (optional)
 
     Returns:
         list of list of int: List of final rewards for all agents for all episodes.
     """
-    e = make(environment, configuration, steps)
+    e = make(environment, configuration, steps, debug=debug, state=state)
     rewards = [[]] * num_episodes
     for i in range(num_episodes):
         last_state = e.run(agents)[-1]
@@ -74,7 +76,8 @@ def make(environment, configuration={}, info={}, steps=[], logs=[], debug=False,
         configuration (dict, optional):
         info (dict, optional):
         steps (list, optional):
-        debug (bool=False, optional):
+        debug (bool=False, optional): Render print() statments to stdout
+        state (optional):
 
     Returns:
         Environment: Instance of a specific environment.
@@ -158,7 +161,7 @@ class Environment:
         else:
             self.reset()
 
-    def step(self, actions, logs=None):
+    def step(self, actions, logs=[]):
         """
         Execute the environment interpreter using the current state and a list of actions.
 
@@ -193,10 +196,10 @@ class Environment:
                 else:
                     action_state[index]["action"] = data
 
-        self.state = self.__run_interpreter(action_state)
+        self.state = self.__run_interpreter(action_state, logs)
 
         # Max Steps reached. Mark ACTIVE/INACTIVE agents as DONE.
-        if len(self.steps) == self.configuration.episodeSteps - 1:
+        if self.state[0].observation.step >= self.configuration.episodeSteps - 1:
             for s in self.state:
                 if s.status == "ACTIVE" or s.status == "INACTIVE":
                     s.status = "DONE"
@@ -253,7 +256,9 @@ class Environment:
         for agent in self.state:
             agent.status = "INACTIVE"
         # Give the interpreter an opportunity to make any initializations.
-        self.__set_state(self.__run_interpreter(self.state))
+        logs = []
+        self.__set_state(self.__run_interpreter(self.state, logs))
+        self.logs.append(logs)
         # Replace the starting "status" if still "done".
         if self.done and len(self.state) == len(statuses):
             for i in range(len(self.state)):
@@ -279,10 +284,11 @@ class Environment:
             if mode == "ansi":
                 return out
         elif mode == "html" or mode == "ipython":
+            is_playing = get(kwargs, bool, self.done, path=["playing"])
             window_kaggle = {
                 "debug": get(kwargs, bool, self.debug, path=["debug"]),
-                "autoplay": get(kwargs, bool, self.done, path=["autoplay"]),
-                "step": 0 if get(kwargs, bool, self.done, path=["autoplay"]) else (len(self.steps) - 1),
+                "playing": is_playing,
+                "step": 0 if is_playing else len(self.steps) - 1,
                 "controls": get(kwargs, bool, self.done, path=["controls"]),
                 "environment": self.toJSON(),
                 "logs": self.logs,
@@ -301,7 +307,7 @@ class Environment:
             html = f'<iframe srcdoc="{player_html}" width="{width}" height="{height}" frameborder="0"></iframe> '
             display(HTML(html))
         elif mode == "json":
-            return json.dumps(self.toJSON(), sort_keys=True)
+            return json.dumps(self.toJSON(), sort_keys=True, indent=2 if self.debug else None)
         else:
             raise InvalidArgument("Available render modes: human, ansi, html, ipython")
 
@@ -520,10 +526,7 @@ class Environment:
             )
         return data
 
-    def __run_interpreter(self, state):
-        if len(self.logs) == 0:
-            self.logs.append([])
-        log = self.logs[-1]
+    def __run_interpreter(self, state, logs):
         out = None
         err = None
         # Append any environmental logs to any agent logs we collected.
@@ -533,7 +536,16 @@ class Environment:
                     args = [structify(state), self]
                     new_state = structify(self.interpreter(
                         *args[:self.interpreter.__code__.co_argcount]))
-                    for agent in new_state:
+                    new_state[0].observation.step = (
+                        0 if self.done
+                        else len(self.steps)
+                    )
+
+                    for index, agent in enumerate(new_state):
+                        if index < len(logs) and "duration" in logs[index]:
+                            duration = logs[index]["duration"]
+                            overage_time_consumed = max(0, duration - self.configuration.actTimeout)
+                            agent.observation.remainingOverageTime -= overage_time_consumed
                         if agent.status not in self.__state_schema.properties.status.enum:
                             self.debug_print(f"Invalid Action: {agent.status}")
                             agent.status = "INVALID"
@@ -551,7 +563,7 @@ class Environment:
                     out = out_buffer.getvalue()
                     err = err_buffer.getvalue()
                     if out or err:
-                        log.append({
+                        logs.append({
                             "stdout": out[0:max_log_length],
                             "stderr": err[0:max_log_length]
                         })
@@ -573,29 +585,32 @@ class Environment:
                 return ("type must be an integer or number", None)
             reward["type"] = [reward_type, "null"]
 
-        # Allow environments to extend the default configuration.
-        configuration = copy.deepcopy(
-            schemas["configuration"]["properties"])
-
-        for k, v in get(spec, dict, {}, ["configuration"]).items():
-            # Set a new default value.
-            if not isinstance(v, dict):
-                if not has(configuration, path=[k]):
+        # Allow environments to extend various parts of the specification.
+        def extend_specification(source, field_name):
+            field = copy.deepcopy(source[field_name]["properties"])
+            for k, v in get(spec, dict, {}, [field_name]).items():
+                # Set a new default value.
+                if not isinstance(v, dict):
+                    if not has(field, path=[k]):
+                        raise InvalidArgument(
+                            f"Field {field} was unable to set default of missing property: {k}")
+                    field[k]["default"] = v
+                # Add a new field.
+                elif not has(field, path=[k]):
+                    field[k] = v
+                # Override an existing field if types match.
+                elif field[k]["type"] == get(v, path=["type"]):
+                    field[k] = v
+                # Types don't match - unable to extend.
+                else:
                     raise InvalidArgument(
-                        f"Configuration was unable to set default of missing property: {k}")
-                configuration[k]["default"] = v
-            # Add a new configuration.
-            elif not has(configuration, path=[k]):
-                configuration[k] = v
-            # Override an existing configuration if types match.
-            elif configuration[k]["type"] == get(v, path=["type"]):
-                configuration[k] = v
-            # Types don't match - unable to extend.
-            else:
-                raise InvalidArgument(
-                    f"Configuration was unable to extend: {k}")
+                        f"Field {field} was unable to extend: {k}")
+
+            spec[field_name] = field
+
+        extend_specification(schemas, "configuration")
+        extend_specification(schemas["state"]["properties"], "observation")
 
-        spec["configuration"] = configuration
         return process_schema(schemas.specification, spec)
 
     def __agent_runner(self, agents):
@@ -637,21 +652,25 @@ class Environment:
         return structify({"act": act})
 
     def __get_shared_state(self, position):
-        if position == 0:
-            return self.state[0]
-        state = copy.deepcopy(self.state[position])
-
         # Note: state and schema are required to be in sync (apart from shared ones).
         def update_props(shared_state, state, schema_props):
             for k, prop in schema_props.items():
-                if get(prop, bool, path=["shared"], fallback=False):
+                # Hidden fields are tracked in the episode replay but are not provided to the agent at runtime
+                if get(prop, bool, path=["hidden"], fallback=False):
+                    if k in state:
+                        del state[k]
+                elif get(prop, bool, path=["shared"], fallback=False):
                     state[k] = shared_state[k]
                 elif has(prop, dict, path=["properties"]):
                     update_props(shared_state[k], state[k], prop["properties"])
             return state
 
-        return update_props(self.state[0], state, self.__state_schema.properties)
+        return update_props(
+            self.state[0],
+            copy.deepcopy(self.state[position]),
+            self.__state_schema.properties
+        )
 
     def debug_print(self, message):
-        if self.debug or self.configuration.isProduction:
+        if self.debug:
             print(message)
