@@ -54,8 +54,10 @@ def test_single_agent():
             "reward": 0,
             "info": {},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 1,
-                "players_raw": []
+                "players_raw": [],
+                "step": 0
             }
         },
         {
@@ -64,6 +66,7 @@ def test_single_agent():
             "reward": 0,
             "info": {},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 0,
                 "players_raw": []
             }
@@ -78,8 +81,10 @@ def test_single_agent():
             "reward": 0,
             "info": {},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 1,
-                "players_raw": []
+                "players_raw": [],
+                "step": 1
             }
         },
         {
@@ -88,6 +93,7 @@ def test_single_agent():
             "reward": 0,
             "info": {},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 0,
                 "players_raw": []
             }
@@ -103,8 +109,10 @@ def test_single_agent():
             "reward": 100,
             'info': {'debug_info': 'Opponent forfeited. You win.'},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 1,
-                "players_raw": []
+                "players_raw": [],
+                "step": 2
             }
         },
         {
@@ -113,6 +121,7 @@ def test_single_agent():
             "reward": None,
             'info': {'debug_info': 'Invalid number of actions provided: Expected 0, got 1.'},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 0,
                 "players_raw": []
             }
@@ -129,8 +138,10 @@ def test_single_agent():
             "reward": None,
             'info': {'debug_info': 'Invalid action provided: [100].'},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 1,
-                "players_raw": []
+                "players_raw": [],
+                "step": 1
             }
         },
         {
@@ -139,6 +150,7 @@ def test_single_agent():
             "reward": 100,
             'info': {'debug_info': 'Opponent forfeited. You win.'},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 0,
                 "players_raw": []
             }
@@ -157,8 +169,10 @@ def test_single_agent():
             "reward": None,
             'info': {'debug_info': 'Invalid number of actions provided: Expected 1, got 0.'},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 1,
-                "players_raw": []
+                "players_raw": [],
+                "step": 1
             }
         },
         {
@@ -167,6 +181,7 @@ def test_single_agent():
             "reward": 100,
             'info': {'debug_info': 'Opponent forfeited. You win.'},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 0,
                 "players_raw": []
             }
@@ -185,8 +200,10 @@ def test_multi_agent():
             "reward": 0,
             "info": {},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 2,
-                "players_raw": []
+                "players_raw": [],
+                "step": 0
             }
         },
         {
@@ -195,6 +212,7 @@ def test_multi_agent():
             "reward": 0,
             "info": {},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 1,
                 "players_raw": []
             }
@@ -209,8 +227,10 @@ def test_multi_agent():
             "reward": 0,
             "info": {},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 2,
-                "players_raw": []
+                "players_raw": [],
+                "step": 1
             }
         },
         {
@@ -219,6 +239,7 @@ def test_multi_agent():
             "reward": 0,
             "info": {},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 1,
                 "players_raw": []
             }
@@ -233,8 +254,10 @@ def test_multi_agent():
             "reward": None,
             'info': {'debug_info': 'Invalid number of actions provided: Expected 2, got 3.'},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 2,
-                "players_raw": []
+                "players_raw": [],
+                "step": 2
             }
         },
         {
@@ -243,6 +266,7 @@ def test_multi_agent():
             "reward": 100,
             'info': {'debug_info': 'Opponent forfeited. You win.'},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 1,
                 "players_raw": []
             }
@@ -260,8 +284,10 @@ def test_deadline():
             "reward": 0,
             "info": {},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 1,
-                "players_raw": []
+                "players_raw": [],
+                "step": 0
             }
         },
         {
@@ -270,6 +296,7 @@ def test_deadline():
             "reward": 0,
             "info": {},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 0,
                 "players_raw": []
             }
@@ -284,8 +311,10 @@ def test_deadline():
             "reward": None,
             "info": {},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 1,
-                "players_raw": []
+                "players_raw": [],
+                "step": 1
             }
         },
         {
@@ -294,6 +323,7 @@ def test_deadline():
             "reward": 100,
             "info": {'debug_info': 'Opponent forfeited. You win.'},
             "observation": {
+                "remainingOverageTime": 60,
                 "controlled_players": 0,
                 "players_raw": []
             }
